/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/message_file', [
    'io.ox/core/extensions',
    'io.ox/preview/main',
    'io.ox/files/api',
    'io.ox/core/tk/dialogs',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/util',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/roster/main',
    'com.voiceworks/ox-messenger/conversation/views/message',
    'com.voiceworks/ox-messenger/core/vendor/virtual-dom',
    'com.voiceworks/ox-messenger/file/main',
    'com.voiceworks/ox-messenger/file/views/partials',
    'io.ox/core/notifications',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, filePrev, fileApi, dialogs, Aggregator, Util, Session, Roster,
             MessageView, vdom, FileController, FileViewPart, notifications, gt) {
    'use strict';

    var h = vdom.h;

    /**
     * Chat Message View
     * ==========================================
     * this.model = message model
     * this.options.conversation = conversation model
     */
    var ChatFileView = MessageView.extend({
        className: 'messenger--chat-message__line',

        initialize: function () {
            var fileDesc,
                onPermissionsDenied = function(){
                    fileDesc.set('permission_denied', true);
                    Aggregator.trigger('file:permisiondenied', fileDesc);
                },
                onPermissionsApproved = function(){
                    Aggregator.trigger('file:permisionapproved', fileDesc);
                };

            // Initialize variables
            this.messageModel = this.model;
            this.ctxMenuAttributes = {
                type: 'button',
                'data-toggle': 'dropdown',
                'aria-haspopup': true,
                'aria-expanded': false
            };

            // Process the file descriptor permissions
            fileDesc = this.getFileDesc();

            // This will alter the file Descriptor, so the permissions variables
            // are properly initialized
            fileDesc.processUserPermissions(Session.user);

            // If the file is uploaded & permissions not denied yet
            if(fileDesc.get('upload_complete') &&
                !fileDesc.get('permissions_loaded')){
                // Make sure we request the permissions only once
                fileDesc.set('permissions_loaded', true);
                // Try to get the file data to detect if the user has permissions
                FileController
                    .getFileData(fileDesc)
                    .done(function(fileData){
                        // If the file is not shared with anybody
                        if(fileData.object_permissions.length === 0){
                            onPermissionsDenied();
                        }else{
                            onPermissionsApproved();
                        }
                    })
                    .fail(onPermissionsDenied);
            }
        },

        // ------------------------------------------- UI/Render Methods ----------------------------------

        template: function () {
            var timestamp = this.messageModel.get('timestamp'),
                fileDesc = this.getFileDesc(),
                oxFile = this.createOXFile(fileDesc),
                commonClass = 'div.messenger--chat-message__file.unselectable.dropup';

            // When user cancelled the file upload
            if(fileDesc.get('upload_canceled')){
                return h(commonClass + '.is-cancelled',
                    [
                        this.getLinkBrokenIcon()
                    ]);
            }
            // If the user has no permissions
            else if(fileDesc.get('permission_denied')){
                return h(commonClass + '.no-permissions',
                    [
                        this.getLinkBrokenIcon()
                    ]);
            }
            // Upload completed and user has permissions and didn't cancel the upload
            else if(fileDesc.get('upload_complete')){
                // Check if the permissions where loaded
                if(fileDesc.get('permissions_loaded')){
                    var previewClass, previewBgIcon;

                    if(oxFile.supportsPreview()){
                        previewClass = '.has-preview';
                        previewBgIcon = this.getPreviewImgIcon();
                    }else{
                        previewClass = '.has-not-preview';
                        previewBgIcon = '';
                    }

                    return h(commonClass + previewClass,
                        {
                            attributes: {title: fileDesc.get('filename')},
                            ondblclick: $.proxy(this.onViewFile, this)
                        },
                        [
                            previewBgIcon,
                            this.getIconDiv(oxFile),
                            this.getInfoDiv(fileDesc),
                            this.getCtxMenu()
                        ]);

                }
                // While we wait for receiveing the permissions, render a spinner
                else{
                    return h(commonClass, [
                        this.getSpinnerIcon()
                    ]);
                }

            }
            // File is beeing uploaded
            else {
                return h(commonClass, [
                    this.getSpinnerIcon(),
                    this.getCtxMenu()
                ]);
            }
        },

        getSpinnerIcon: function() {
            return h('div.messenger--file-icon.fa.fa-spinner.fa-spin',
                {
                    attributes: this.ctxMenuAttributes
                });
        },

        getPreviewImgIcon: function() {
            var fileTypeIconClass = FileViewPart.getIconClasses(this.getFileDesc(), true);

            return h('div.messenger--file-icon.preview--icon.fa'+fileTypeIconClass,
                {
                    attributes: this.ctxMenuAttributes
                });
        },

        getLinkBrokenIcon: function() {
            return h('div.messenger--file-icon.fa.fa-chain-broken');
        },

        getIconDiv: function(oxFile) {
            if(oxFile.supportsPreview())
                return this.getPreviewDiv(oxFile);
            else
                return this.getFileIcon();
        },

        getFileIcon: function(){
            var className = FileViewPart.getIconClasses(this.getFileDesc(), true);

            return h('div.messenger--file-icon.fa'+className,
                {
                    attributes: this.ctxMenuAttributes, // Makes it clickable to pop up the dropdown menu
                }
            );
        },

        getPreviewDiv: function(oxFile){
            var options = {
                    scaletype: 'auto',
                    width: 800, // Aprox 34kb
                    height: 800
                },
                previewUrl = fileApi.getUrl(oxFile.toJSON(), 'thumbnail', options);

            return h('div.messenger--file-preview',
                {
                    attributes: this.ctxMenuAttributes, // Makes it clickable to pop up the dropdown menu
                    style: {'background-image': 'url('+previewUrl+')'}
                }
            );
        },

        getInfoDiv: function(fileDesc) {
            var fileName = fileDesc.get('filename'),
                fileSize = Util.formatFileSize(fileDesc.get('file_size')),
                creationDate = Util.formatTime(this.messageModel.get('timestamp'));

            return h('div.messenger--file-data.dropdown-toggle',
                {
                    attributes: this.ctxMenuAttributes // Makes it clickable to pop up the dropdown menu
                },
                [
                    h('div.messenger--h5',
                        { innerHTML: fileName}),
                    h('div.messenger--h6.messenger--text-color--light-gray',
                        { innerHTML: fileSize}),
                    h('div.messenger--h6.messenger--text-color--light-gray',
                        { innerHTML: creationDate})
                ]);
        },

        getCtxMenu: function(){
            var options = [],
                fileDesc = this.getFileDesc();

            if(fileDesc.get('upload_complete')) {
                options.push(h('li', [h('a', {innerHTML: gt('View file'), 'onclick': $.proxy(this.onViewFile, this)})]));
                options.push(h('li', [h('a', {innerHTML: gt('Download file'), 'onclick': $.proxy(this.onDownloadFile, this)})]));
                if(!fileDesc.get('is_shared'))
                    options.push(h('li', [h('a', {innerHTML: gt('Cancel share'), 'onclick': $.proxy(this.onCancelShare, this)})]));
            } else{
                options.push(h('li', [h('a', {innerHTML: gt('Cancel upload'), 'onclick': $.proxy(this.onCancelUpload, this)})]));
            }

            return h('ul.dropdown-menu', options);
        },

        getFileDesc: function() {
            return this.messageModel.get('fileDesc');
        },

        createOXFile: function(fileDesc){
            var previewData = fileDesc.toPreviewData(),
                oxFile = new fileApi.Model(previewData);

            return oxFile;
        },

        // ------------------------------------------- UI Callbacks ----------------------------------

        onViewFile: function () {
            var fileDesc = this.getFileDesc();

            // Make sure we have access to the file
            FileController
                .getFileData(fileDesc)
                .done(function(){
                    ox.load(['io.ox/core/viewer/main']).done(function (Viewer) {
                        var viewer = new Viewer();

                        viewer.launch({ selection: fileDesc.toObject(true), files: [fileDesc.toObject(true)]});
                    });
                })
                .fail(function(){
                    notifications.yell('warning', gt('The access to this file has been revoked by the owner.'));
                });
        },

        onDownloadFile: function () {
            var fileDesc = this.getFileDesc();

            // Make sure we have access to the file
            FileController
                .getFileData(fileDesc)
                .done(function(){
                    ox.load(['io.ox/files/actions/download']).done(function (action) {
                        action([fileDesc.toObject(true)]);
                    });
                })
                .fail(function(){
                    notifications.yell('warning', gt('The access to this file has been revoked by the owner.'));
                });
        },

        onCancelShare: function() {
            var fileDesc = this.getFileDesc();

            new dialogs.ModalDialog()
                .text(gt('Do you really want to stop sharing this item?'))
                .addPrimaryButton('delete', gt('Yes'), 'delete',  { 'tabIndex': '1' })
                .addButton('cancel', gt('No'), 'cancel',  { 'tabIndex': '1' })
                .show()
                // Check user answer before cancelling
                .then(function (action) {
                    if (action === 'delete') {
                        return FileController.cancelShareFile(fileDesc);

                        // In case we need to open a file permissions dialog
                        // 'io.ox/files/share/permissions' ->  filePermissions
                        // FileController
                        //    .getFileShareData(this.getFileDesc())
                        //    .then(function(data){
                        //        filePermissions.show(data);
                        //    });
                    }
                    return false;
                })
                // If sharing was cancelled, trigger event
                .then(function(shareCanceled){
                    if(shareCanceled){
                        fileDesc.set('permission_denied', true);
                        Aggregator.trigger('file:permisiondenied', fileDesc);
                    }
                });
        },

        onCancelUpload: function() {
            this.getFileDesc().set('upload_canceled', true);
            Aggregator.trigger('file:uploadcancelled', this.getFileDesc());
        }
    });

    return ChatFileView;
});
