/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/messageform', [
    'io.ox/core/extensions',
    'io.ox/core/notifications',
    'io.ox/files/api',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/conversation/models/message',
    'com.voiceworks/ox-messenger/conversation/models/conversation',
    'com.voiceworks/ox-messenger/conversation/models/message/factory',
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/file/main',
    'com.voiceworks/ox-messenger/file/views/upload-button',
    'com.voiceworks/ox-messenger/util',
    'settings!io.ox/core',
    'gettext!com.voiceworks/ox-messenger'
], function (ext, notifications, fileApi, Aggregator, Session, MessageModel, ConversationModel,
             MessageFactory, DisposableView, FileController, UploadButton, Utils, Settings, gt) {
    'use strict';

    /**
     * Message Form View
     */
    var MessageFormView = DisposableView.extend({

        className: 'messenger--chat-input',

        events: {
            'keydown textarea': 'handleKeydown',
            'keyup textarea': 'handleKeyup',
            'click .messenger--chat-input__send': 'onClickSend'
        },

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.user = options.user;
            this.model = this.model || new ConversationModel();

            this.model.on('change:whosComposingText', $.proxy(this.onComposingParticipantsChange, this));

            ox.on('connection:offline', $.proxy(this.disableView, this));
            ox.on('connection:online', $.proxy(this.enableView, this));
            ox.on('relogin:required', $.proxy(this.disableView, this));
        },

        onClickSend: function (e) {
            e.preventDefault();
            var messageText = $.trim($('.messenger--chat-input__textarea').val());
            if (messageText === '') {
                return;
            }
            this.sendMessage(messageText);
        },

        onComposingParticipantsChange: function(){
            if (this.$el) {
                this.$el
                    .find('.messenger--chat-input__composing')
                    .text(this.model.get('whosComposingText'));
            }
        },

        handleKeyup: function (e) {
            var inputContainer = this.$el;
            var chatContainer  = $('.messenger--chat-messages');
            var inputText      = $.trim($('.messenger--chat-input__textarea').val());

            $(e.target).removeClass('two-lines three-lines four-lines');
            inputContainer.removeClass('two-lines three-lines four-lines');
            chatContainer.removeClass('two-lines three-lines four-lines');

            if (e.target.scrollHeight >= 80) {
                $(e.target).addClass('four-lines');
                inputContainer.addClass('four-lines');
                chatContainer.addClass('four-lines');

            } else if (e.target.scrollHeight >= 60) {
                $(e.target).addClass('three-lines');
                inputContainer.addClass('three-lines');
                chatContainer.addClass('three-lines');

            } else if (e.target.scrollHeight >= 40) {
                inputContainer.addClass('two-lines');
                chatContainer.addClass('two-lines');
                $(e.target).addClass('two-lines');
            }

            // If there's text at this moment, set to Composing
            if(inputText.length > 0) {
                this.model.setChatState(ConversationModel.STATE_COMPOSING);
            }
            // If there's no text, set to Paused
            else {
                this.model.setChatState(ConversationModel.STATE_PAUSED);
            }
        },

        sendMessage: function (messageText) {
            var maxMessageLen = 10000;

            // Check the length of the string (max size supported by XMPP is 16384 bytes)
            // If Len > maxMessageLen -> split recursively
            if(messageText.length > maxMessageLen) {
                this.sendMessage(messageText.substr(0, maxMessageLen));
                this.sendMessage(messageText.substring(maxMessageLen));
                return;
            }

            // Check if you are a participant
            if (!this.model.participants.get(this.model.user.id)) {
                notifications.yell('warning', gt('You are not participating in this conversation. Rejoin this conversation to send messages.'));
                return;
            }

            if (Session.get('connection') === 'offline' || Session.get('server_xmpp') === 'offline') {
                notifications.yell('warning', gt('You can not send messages. You are not connected to the chat server.'));
                return;
            }

            $('.messenger--chat-input__textarea').val('');
            $('.messenger--chat-input__textarea').trigger('keyup');

            var message = MessageFactory.create({
                username     : this.user.id,
                from         : this.user.id,
                message      : messageText,
                timestamp    : new Date().getTime(),
                isHistory    : false,
                conversation : this.model
            });

            this.model.addMessage(message);

            Aggregator.trigger('message:send', message);

            this.model.setChatState(ConversationModel.STATE_ACTIVE);
        },

        handleKeydown: function (e) {
            if (!(e.which === 13 && !e.shiftKey)) {
                return;
            }
            e.preventDefault();
            e.stopImmediatePropagation();
            var messageText = $.trim($('.messenger--chat-input__textarea').val());
            if (messageText === '') {
                return;
            }
            this.sendMessage(messageText);
        },

        focusInput: function (initiatedFromUI) {
            setTimeout(function () {
                this.$('.messenger--chat-input__textarea').focus();
            }, initiatedFromUI ? 150 : 0); // time to let the user click
        },

        render: function () {
            var fileUploadBt         = '',
                fileUploadBtRendered = '',
                properties           = Settings.get('properties');

            // If Drive is available && there's free space
            if (FileController.spaceAvailable()) {
                fileUploadBt = new UploadButton();
                fileUploadBtRendered = fileUploadBt.render().$el;
            }

            this.$el.empty();
            this.$el.append(
                $('<div class="messenger--chat-input__composing  messenger--text-color--light-gray  messenger--h6">')
                    .text(this.model.get('whosComposingText')),
                $('<div class="messenger--table  messenger--table__chat-input">').append(
                    fileUploadBtRendered,
                    $('<div class="messenger--table__cell  messenger--table__cell--chat-input__input">').append(
                        $('<textarea class="messenger--chat-input__textarea">').attr('placeholder', gt('Send a message'))
                    ),
                    $('<div class="messenger--table__cell  messenger--table__cell--chat-input__send">').append(
                        $('<button class="btn btn-primary sendMessage control f6-target messenger--chat-input__send">').attr({
                            'data-action' : 'send',
                            'tabindex'    : '1'
                        }).text(gt('Send'))
                    )
                )
            );

            if (this.model.isMuc()) {
                this.$el.addClass('messenger--chat-input--muc');
            }

            return this;
        },

        disableView: function () {
            if (!this.$el) { return; }

            this.$el.find('.messenger--chat-input__send').addClass('disabled');
            this.$el.find('.messenger--chat-input__textarea').prop('disabled', true);

            this.undelegateEvents();
        },

        enableView: function () {
            if (!this.$el) { return; }

            this.$el.find('.messenger--chat-input__send').removeClass('disabled');
            this.$el.find('.messenger--chat-input__textarea').prop('disabled', false);

            this.delegateEvents();
        },
    });

    return MessageFormView;
});
