/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/conversation/views/stream', [
    'com.voiceworks/ox-messenger/core/views/disposable',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/views/partials/callpart',
    'com.voiceworks/ox-messenger/core/views/timer'
], function (DisposableView, Aggregator, CallPart, TimerView) {
    'use strict';

    var StreamView = DisposableView.extend({
        className: 'messenger--stream',

        initialize: function (options) {
            DisposableView.prototype.initialize.apply(this, arguments);

            this.id    = this.model.id;
            this.title = options.title || this.model.getDisplayName();
            this.call  = options.call || this.model.call;

            this.listenTo(this.call, 'change:callState', this.stateCheck);
            this.listenTo(this.model, 'change:streamFrozen', this.showFrozen);
        },

        render: function () {
            var streamTitle = this.title,
                streamStatus = this.call.get('callState'),
                streamMedia,
                streamFrozen,
                streamAvatar,
                streamTitle,
                streamStatus,
                streamThrobber;

            this.$el.children().remove();
            this.$el.prevObject = null;
            this.$el.attr('id', 'stream-' + this.model.id.replace(/[@|.]/g, '_'));

            streamMedia    = $('<div class="messenger--stream__media messenger--is-hidden">');
            streamFrozen   = $('<div class="messenger--stream__frozen messenger--is-hidden">').append(
                $('<i class="fa fa-spinner fa-spin">')
            );
            streamAvatar   = $('<div class="messenger--avatar messenger--stream__avatar">').css({
                'background-image' : 'url(' + this.model.getImageUrl() + ')'
            });
            streamTitle    = $('<div class="messenger--stream__title messenger--h4">').text(streamTitle);
            streamStatus   = $('<div class="messenger--stream__status">').text(streamStatus);
            streamThrobber = CallPart.getConnectingAnimation().addClass('messenger--stream__throbber messenger--is-hidden');

            this.$el.append(
                streamMedia,
                streamFrozen,
                streamAvatar,
                streamTitle,
                streamStatus,
                streamThrobber
            );

            // Update View
            this.stateCheck();

            return this;
        },

        stateCheck: function () {
            if (!this.model) { return; }

            var callState = this.call.get('callState');

            switch (callState) {
                case 'idle':
                case 'calling':
                    this.stateIsIdle();
                    break;
                case 'in-progress':
                    this.stateIsInProgress();
                    break;
                case 'group-initiated':
                    this.stateIsGroupInitiated();
                    break;
                case 'answered':
                    this.stateIsAnswered();
                    break;
            }
        },

        // idle: hangup
        // hide all
        // show nothing
        stateIsIdle: function () {
            this.$el.find('.messenger--stream__media video').remove();

            this.showMedia(false);
            this.showAvatar(true);
            this.showStatus(true);
            this.showThrobber(true);

            this.model.set('lastCallFrozen', this.model.get('streamFrozen'));
            this.model.set('streamFrozen', false);
        },

        // in-progress: not yet answered
        // Show Ringing status
        stateIsInProgress: function () {
            this.showMedia(false);
            this.showAvatar(true);
            this.showStatus(true);
            this.showThrobber(true);
        },

        // group-initiated: Only when it's my stream on a group audio call
        stateIsGroupInitiated: function () {
            this.showMedia(false);
            this.showAvatar(true);
            this.showStatus(true);
            this.showThrobber(true);
        },

        // answered: call started
        // show stream and add video to stream, and show timer
        // hide icon and status
        stateIsAnswered: function () {
            this.media = this.call.get('callSession').getMediaCanvas();
            var media          = this.media,
                videoElem      = this.$el.find('.messenger--stream__media'),
                streamId       = 'video-stream-' + this.model.id.replace(/[@|.]/g, '_'),
                tmpVideoStream = $('#' + streamId),
                callType       = this.call.get('type');

            var onCanPlay = function () {
                // When Group Video Call
                if (callType === 'video' || callType === 'groupvideo') {
                    this.showMedia(true);
                    this.showAvatar(false);
                }
                // When Group Audio Call
                else if (callType === 'audio' || callType === 'groupaudio') {
                    this.showMedia(false);
                    this.showAvatar(true);
                }

                this.showStatus(false);
                this.showThrobber(false);

                // Listen to the 'canplay' to wait for media
                media.remote.removeEventListener('canplay', onCanPlay);
            };

            // There is a video saved, and it's a MUC
            if (tmpVideoStream.length) {
                media.remote = tmpVideoStream.get(0);
                tmpVideoStream.remove();
            }
            media.remote.setAttribute('id', streamId);
            media.remote.setAttribute('class', 'groupStreamVideo');

            if (media.remote && (callType === 'video' || callType === 'groupvideo')) {
                this.progressListener   = this.progressHandler.bind(this);
                this.timeupdateListener = this.timeupdateHandler.bind(this);
                $(media.remote)
                    .on('progress', this.progressListener)
                    .on('timeupdate', this.timeupdateListener);
            }

            videoElem.empty().append(media.remote);

            // Listen to the 'canplay' to wait for media
            media.remote.addEventListener('canplay', $.proxy(onCanPlay, this), false);

            if (media.remote.paused) {
                setTimeout(function () {
                    media.remote.play();
                }, 100);
            }

            Aggregator.trigger('groupcall:answered', {
                call : this.call,
                user : this.model
            });
        },

        showMedia: function (show) {
            var elem = this.$el.find('.messenger--stream__media');
            this.showElement(elem, show);
        },

        showAvatar: function (show) {
            var elem = this.$el.find('.messenger--stream__avatar');
            this.showElement(elem, show);
        },

        showStatus: function (show) {
            var elem = this.$el.find('.messenger--stream__status');
            this.showElement(elem, show);
            this.$el.find('.messenger--stream__status').text(this.call.getCallStateDisplayName());
        },

        showThrobber: function (show) {
            var elem = this.$el.find('.messenger--stream__throbber');
            this.showElement(elem, show);
            this.$el.find('.messenger--stream__status').text(this.call.getCallStateDisplayName());
        },

        showElement: function (elem, show) {
            if (!elem) { return; }
            show ? elem.removeClass('messenger--is-hidden') : elem.addClass('messenger--is-hidden');
        },

        showFrozen: function () {
            var frozen = this.model.get('streamFrozen');
            var elem = this.$el.find('.messenger--stream__frozen');
            this.showElement(elem, frozen);
        },

        timeupdateHandler: function (event) {
            if (this.model) {
                this.model.set('streamCurrentTime', new Date().getTime());
            }
        },

        progressHandler: function (event) {
            var tmpTime        = new Date().getTime(),
                tmpFrozenLimit = 2000,
                frozen         = this.model.get('streamFrozen'),
                currentTime    = this.model.get('streamCurrentTime');

            if ((tmpTime - currentTime) > tmpFrozenLimit && !frozen && currentTime !== 0) {
                this.model.set('streamFrozen', true);
            }
            else if ((tmpTime - currentTime) <= tmpFrozenLimit && frozen && currentTime !== 0) {
                this.model.set('streamFrozen', false);
            }
        },


        removeProgressListener: function () {
            if (this.media) {
                $(this.media.remote)
                    .off('progress', this.progressListener)
                    .off('timeupdate', this.timeupdateListener);
                delete this.media;
            }
        },

        dispose: function() {
            // the super call
            DisposableView.prototype.dispose.call(this);
            this.removeProgressListener();
        }
    });

    return StreamView;
});
