/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

/*globals UCConnect*/
define('com.voiceworks/ox-messenger/main', [
    'io.ox/core/extensions',
    'io.ox/core/commons',
    'io.ox/core/page-controller',
    'io.ox/core/tk/list-control',
    'com.voiceworks/ox-messenger/core/heap-counter',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/core/session',
    'com.voiceworks/ox-messenger/conversation/main',
    'com.voiceworks/ox-messenger/conversation/toolbar',
    'io.ox/core/notifications',
    'gettext!com.voiceworks/ox-messenger',
    'settings!com.voiceworks/ox-messenger',
    'less!io.ox/mail/style'
], function (coreExt, Commons, PageController, ListViewControl, HeapCounter, Aggregator, Session, Conversation, Toolbar, CoreNotification, gt, settings) {

    'use strict';

    var app = ox.ui.createApp({
        name: 'com.voiceworks/ox-messenger',
        title: 'Messenger'
    });

    app.mediator({

        'pages-desktop': function (app) {
            if (_.device('small')) {
                return;
            }

            app.pages = new PageController(app);

            var nodesMain = app.getWindow().nodes.main;

            // Add Messenger Namespace
            nodesMain.addClass('io-ox-messenger');

            // create 2 pages
            // legacy compatibility
            nodesMain.addClass('vsplit');

            app.pages.addPage({
                name: 'listView',
                container: nodesMain,
                classes: 'leftside'
            });
            app.pages.addPage({
                name: 'detailView',
                container: nodesMain,
                classes: 'rightside'
            });
        },

        'vsplit': function (app) {
            // replacing vsplit with new pageController
            // TODO: refactor app.left and app.right
            var left = app.pages.getPage('listView'),
                right = app.pages.getPage('detailView');

            app.left = left.addClass('messenger messenger--left  border-right');

            app.right = right.addClass('messenger messenger--right').attr({
                'role': 'complementary',
                //#. Aria label for Messenger right side
                'aria-label': gt('Messenger Conversation')
            });
        },

        'props': function (app) {
            // introduce shared properties
            app.props = new Backbone.Model({
                'layout': _.device('smartphone') ? 'vertical' : app.settings.get('layout', 'vertical'),
                'checkboxes': _.device('smartphone') ? false : app.settings.get('showCheckboxes', true),
                'contactPictures': app.settings.get('showContactPictures', false)
            });
        },

        // Selection message
        'selection-message': function (app) {
            app.conversationNode = app.right.append(
                $('<div class="io-ox-center multi-selection-message">').append(
                    $('<div>').text(
                        gt('No conversation selected')
                    )
                )
            );
        },

        'conversation': function (app) {
            Conversation.init(app);
        },

        'conversation-view': function (app) {
            Conversation.initConversation(app.conversationNode);
        },

        'list-view': function (app) {
            if (_.device('small')) {
                return;
            }
            Conversation.initList(app.left);
            Conversation.showList();
        },

        // Setup list view control
        'list-view-control': function (app) {
            if (_.device('small')) {
                return;
            }
            app.listControl = new ListViewControl({
                id: 'com.voiceworks/ox-messenger',
                listView: Conversation.listView,
                app: app
            });
            app.left.append(app.listControl.render().$el);
            app.listControl.$el.addClass('toolbar-top-visible');
            app.listControl.resizable();
        },

        // Respond to global refresh
        'refresh': function (app) {
            Conversation.listView && Conversation.listView.reload();
        },


        // Respond to single and multi selection in list view
        'selection': function (app) {
            if (_.device('small')) {
                return;
            }

            function resetRight(className) {
                return app.right
                    .removeClass('selection-empty selection-one selection-multiple preview-visible'.replace(className, ''))
                    .addClass(className);
            }

            var react = _.debounce(function (type, list) {
                switch (type) {
                case 'empty':
                    resetRight('selection-empty');
                    break;
                case 'one':
                case 'action':
                    resetRight('selection-one');
                    break;
                }
            }, 100);

            Conversation.listView && Conversation.listView.on({
                'selection:empty': function () {
                    react('empty');
                },
                'selection:one': function (list) {
                    react('one', list);
                },
                'selection:action': function (list) {
                    // make sure we are not in multi-selection
                    if (Conversation.listView && Conversation.listView.selection.get().length === 1) {
                        react('action', list);
                    }
                }
            });
        },

        /*
         * Respond to changed sort option
         */
        'change:sort': function (app) {
            app.props.on('change:sort', function (model, value) {
                var tmpSelection = app.listControl.listView.selection.get();
                app.props.set('sort', value);
                app.listControl.listView.collection.setSortMethod(app.props.get('sort'), app.props.get('order'));
                app.listControl.listView.collection.sort();
                app.listControl.listView.collection.trigger('reset');
                app.listControl.listView.selection.set(tmpSelection);
            });
        },

        /*
         * Respond to changed sort option
         */
        'change:order': function (app) {
            app.props.on('change:order', function (model, value) {
                var tmpSelection = app.listControl.listView.selection.get();
                app.props.set('order', value);
                app.listControl.listView.collection.setSortMethod(app.props.get('sort'), app.props.get('order'));
                app.listControl.listView.collection.sort();
                app.listControl.listView.collection.trigger('reset');
                app.listControl.listView.selection.set(tmpSelection);
            });
        },

        /*
         * Get folder-based view options
         */
        'get-view-options': function (app) {
            app.getViewOptions = function () {
                var options = app.settings.get('viewOptions');
                return _.extend({ sort: 'timestamp', order: 'desc' }, options);
            };
        },

        /*
         * Store view options
         */
        'store-view-options': function (app) {
            app.props.on('change', _.debounce(function () {
                var data = app.props.toJSON();
                app.settings
                    .set('viewOptions', { sort: data.sort, order: data.order })
                    .set('showCheckboxes', data.checkboxes)
                    .set('showContactPictures', data.contactPictures)
                    .save();
            }, 500));
        },

        /*
         * Restore view opt
         */
        'restore-view-options': function (app) {
            var data = app.getViewOptions();
            app.props.set(data);
        },

        /*
         * Respond to change:checkboxes
         */
        'change:checkboxes': function (app) {
            if (_.device('small')) {
                return;
            }
            app.props.on('change:checkboxes', function (model, value) {
                Conversation.listView && Conversation.listView.toggleCheckboxes(value);
            });
        },

        /*
         * Respond to change:checkboxes on mobiles
         * Change "edit" to "cancel" on button
         */
        'change:checkboxes-mobile': function (app) {
            if (_.device('!small')) {
                return;
            }
            app.props.on('change:checkboxes', function (model, value) {
                Conversation.listView && Conversation.listView.toggleCheckboxes(value);
                if (value) {
                    app.pages.getNavbar('listView')
                        .setRight(gt('Cancel'))
                        .hide('.left');
                } else {
                    app.pages.getNavbar('listView')
                        .setRight(gt('Edit'))
                        .show('.left');
                    // reset navbar title on cancel
                    app.folder.getData().done(function (d) {
                        app.pages.getCurrentPage().navbar.setTitle(d.title);
                    });

                    // reset selection
                    Conversation.listView && Conversation.listView.selection.selectNone();
                }
            });
        },

        'list-view-checkboxes': function (app) {
            // always hide checkboxes on small devices initially
            if (_.device('small')) {
                return;
            }
            /*app.UCConnect.rosterDeferred.done(function () {
                app.listView.toggleCheckboxes(app.props.get('checkboxes'));
            });*/
            Conversation.listView.toggleCheckboxes(app.props.get('checkboxes'));
        },

        'list-view-checkboxes-mobile': function (app) {
            // always hide checkboxes on small devices initially
            if (!_.device('small')) {
                return;
            }
            app.props.set('checkboxes', false);
            /*app.UCConnect.rosterDeferred.done(function () {
                app.listView.toggleCheckboxes(false);
            });*/
            Conversation.listView.toggleCheckboxes(false);
        },

        'show-conversation': function (app) {
            app.showConversation = function (conversation, dontTrigger) {
                return Conversation.showConversation(conversation, dontTrigger);
            };
        },

        'handle-incoming-message': function (app) {
            app.handleIncomingMessage = function (data) {
                Session.user.handleMessage(data);
            };
        },

        // Respond to change:contactPictures
        'change:contactPictures': function (app) {
            app.props.on('change:contactPictures', function () {
                Conversation.listView.redraw();
            });
        },

        // Respond to changing layout
        'apply-layout': function (app) {

            app.applyLayout = function () {

                var layout = app.props.get('layout'),
                    nodes = app.getWindow().nodes,
                    toolbar,
                    className;

                if (layout === 'compact') {
                    nodes.main.addClass('preview-right').removeClass('preview-bottom preview-none');
                    nodes.main.find('.messenger--window-right-container').addClass('messenger--window-right-container--compact');

                } else if (layout === 'vertical') {
                    nodes.main.addClass('preview-right').removeClass('preview-bottom preview-none');
                    nodes.main.find('.messenger--window-right-container').removeClass('messenger--window-right-container--compact');
                }

                // remove inline styles from using the resize bar
                app.left.css({ width: '', height: '' });
                app.right.css({ left: '', top: '' });

                // relocate toolbar
                toolbar = nodes.body.find('.classic-toolbar');
                className = 'classic-toolbar-visible';
                if (layout === 'compact') {
                    nodes.body.removeClass(className);
                    app.right.addClass(className).prepend(toolbar);
                } else {
                    app.right.removeClass(className);
                    nodes.body.addClass(className).prepend(toolbar);
                }
            };

            app.props.on('change:layout', function () {
                app.applyLayout();
                Conversation.listView.redraw();
            });

            app.getWindow().on('show:initial', function () {
                app.applyLayout();
            });
        }
    });

    //launcher
    app.setLauncher(function () {

        // get window
        var win = ox.ui.createWindow({
            name: 'com.voiceworks/ox-messenger',
            title: 'Messenger',
            chromeless: true
        });

        if (_.url.hash('debug-messenger') && HeapCounter.isAvailable()) {
            var counter = new HeapCounter($('#io-ox-topbar')[0]);
        }

        win.on('show', function () {
            // Delay the badge update to allow the menu item to replace
            setTimeout(function () {
                Session.user.conversations.updateAppUnreadBadge();
                Session.user.conversations.updateAppUnreadActiveConversation();
                $('.messenger--chat-input__textarea:visible').eq(0).focus();
            }, 200);
        });

        app.setWindow(win);
        app.settings = settings;

        app.mediate();
        win.setTitle(gt('Messenger'));
        win.show();

        if (!Session.get('xmppHistoryLoaded')) {
            app.left.busy(false);
        }

        if (UCConnect.capabilities.mustFlash) {
            CoreNotification.yell('warning', gt('Currently we only support audio and video calls in Chrome and Firefox browsers. Please switch to one of those browsers to have audio and video calls.'));
        }

        Aggregator.on('xmpp-history:loaded', function () {
            setTimeout(function () {
                app.left.idle();
            }, 500);

        });

    });

    return {
        getApp: app.getInstance
    };
});
