/**
 * This work is provided under the terms of the CREATIVE COMMONS PUBLIC
 * LICENSE. This work is protected by copyright and/or other applicable
 * law. Any use of the work other than as authorized under this license
 * or copyright law is prohibited.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * © 2015 Voiceworks B.V. info@voiceworks.com
 *
 */

define('com.voiceworks/ox-messenger/roster/main', [
    'io.ox/core/api/user',
    'com.voiceworks/ox-messenger/aggregator',
    'com.voiceworks/ox-messenger/roster/views/roster',
    'com.voiceworks/ox-messenger/roster/models/groupcollection',
    'com.voiceworks/ox-messenger/roster/models/group',
    'com.voiceworks/ox-messenger/roster/models/usercollection',
    'com.voiceworks/ox-messenger/core/user',
    'com.voiceworks/ox-messenger/core/views/partials/userpart',
    'com.voiceworks/ox-messenger/util',
    'gettext!com.voiceworks/ox-messenger',
    'less!com.voiceworks/ox-messenger/style',
    'less!com.voiceworks/ox-messenger/phone/style'
], function (userApi, Aggregator, RosterView, RosterGroupCollection, RosterGroupModel, RosterUserCollection, UserModel,
             UserPart, Util, gt) {
    'use strict';

    /**
     * Main Roster Application
     */
    var Roster = {
        init: function (user) {
            // Remove old toggle listener
            Aggregator.off('roster:toggle');
            Aggregator.on('roster:toggle', $.proxy(this.toggleHandler, this));
            Aggregator.on('session:disconnected', $.proxy(this.offlineHandler, this));
            Aggregator.on('roster:hide', $.proxy(this.hide, this));

            this.user = user;
            this.groups = new RosterGroupCollection();
            this.created = false;

            // Create default Buddylist group
            var buddylist = new RosterGroupModel({
                id: 1,
                name: gt('Buddylist'),
                index: 9000 // It's over 9000! Put at bottom
            });

            var favorites = new RosterGroupModel({
                id: 2,
                name: gt('Favorites'),
                index: 100 // Put at the top
            });

            var guestUsers = new RosterGroupModel({
                id: 3,
                name: gt('Guest Users'),
                index: 150 // Put below Favorites
            });

            this.groups.add([buddylist, favorites, guestUsers]);

            this.view = new RosterView({ user: user, collection: this.groups });
            require('io.ox/core/notifications').on('show', $.proxy(this.hide, this));
        },

        attach: function () {
            $('#io-ox-core').prepend(
                $('<div class="messenger--roster messenger--is-hidden" tabindex="-1">'),
                $('<div class="abs messenger--roster-overlay messenger--is-hidden">').click($.proxy(this.hide, this))
            );
        },

        hasUser: function (id) {
            var theSession = require('com.voiceworks/ox-messenger/core/session');

            id = Util.getBareJid(id);

            if (Util.isSameJid(theSession.user.id, id)) {
                return theSession.user;
            }

            return this.getAllUsers().get(id);
        },

        getUserById: function (id) {
            // @TODO refactor this, just refactor handleMessage from sessionuser
            var Session = require('com.voiceworks/ox-messenger/core/session');

            id = Util.getBareJid(id);

            if (Util.isSameJid(Session.user.id, id)) {
                return Session.user;
            }

            return this.getAllUsers().get(id) || new UserModel({ id: id, inRoster: false });
        },

        getAllUsers: function () {
            return this.groups.get(1).users;
        },

        getNextUnusedId: function () {
            var object = this.getGroupsObject();

            if (!object) {
                return false;
            }

            return _.size(object) + 100;
        },

        getGroups: function () {
            return this.groups;
        },

        getGroup: function (id) {
            return this.groups.get(id);
        },

        getGroupsObject: function () {
            if (!this.groups) {
                return;
            }

            var that   = this,
                object = {};

            this.groups.each(function (group) {
                var groupId = Util.keyify(group.id);
                object[groupId] = {
                    name: group.get('name'),
                    toggled: group.get('toggled')
                };
            });

            return object;
        },

        getGroupKeys: function () {
            var object = this.getGroupsObject();

            if (!object) {
                return;
            }

            var objectKeys =  _.keys(object);
            var objectKeysClean = _.map(objectKeys, function(key) {
                return key.toLowerCase().replace(/\W/g, '_');
            });

            return objectKeysClean;
        },

        getGroupValues: function () {
            var object = this.getGroupsObject();

            if (!object) {
                return;
            }

            return _.values(object);
        },

        getGroupToggleStatuses: function () {
            var groups              = this.getGroups(),
                groupToggleStatuses = {};

            groups.each(function (group) {
                groupToggleStatuses[group.id] = group.get('toggled');
            });

            return groupToggleStatuses;
        },

        getGroupUsers: function (id) {
            var group = this.groups.get(id);

            if (!group) {
                return null;
            }

            return group.users;
        },

        create: function (buddies) {
            var that            = this,
                contacts        = this.user.contacts,
                buddylistGroup  = this.groups.get(1),
                favoritesGroup  = this.groups.get(2),
                guestUserGroup  = this.groups.get(3),
                groupsToAdd     = [],
                groupsObj       = {
                    1: [],
                    2: [],
                    3: []
                };

            // Build Roster
            _.each(buddies, function (buddy, jid) {
                var user    = that.getUserById(jid),
                    contact = _.find(contacts, function (contact) {
                        return contact.email1 === jid;
                    });

                // Update user model
                user.set(buddy);
                user.set({
                    id      : jid,
                    contact : contact || {}
                });

                // Set subscription
                // @QUESTION: What is subscription?
                if (user.get('subscription') !== buddy.subscription) {
                    user.set({
                        'subscription': buddy.subscription
                    });
                }

                // Add all users to the Buddylist
                groupsObj[1].push(user);

                // Check for groups
                if (buddy.groups.length === 0) {
                    return;
                }

                // Add users to groups
                //
                // 1: Buddylist (reserved ID)
                // 2: Favorites
                // 3: Guest Users
                // 3+: Custom Group
                _.each(buddy.groups, function (groupId) {
                    var groupModel = '';

                    // Add the groupId to the Properties of this User
                    user.properties.addProperty(groupId, true);

                    // Create group if it does not exist
                    if (groupsObj[groupId] === undefined) {
                        var newGroupId    = parseInt(groupId, 10),
                            newGroupName  = 'Custom Group',
                            newGroupIndex = 200;

                        if (newGroupId !== 1 || newGroupId !== 2 || newGroupId !== 3) {
                            var groupModel = that.createGroup(newGroupName, newGroupId, newGroupIndex);

                            groupsToAdd.push(groupModel);

                            groupsObj[groupId] = [];
                        }
                    }

                    groupsObj[groupId].push(user);
                });
            });

            // Add buddylist group (1)
            groupsToAdd.push(buddylistGroup);
            groupsToAdd.push(favoritesGroup);
            groupsToAdd.push(guestUserGroup);

            this.groups.reset(groupsToAdd);

            _.each(groupsObj, function (users, groupId) {
                var group = that.groups.get(parseInt(groupId, 10));
                group.users.reset(users);
            });

            Aggregator.trigger('roster:created', true);
        },

        createGroup: function (name, id, index) {
            var id        = id ? id : this.getNextUnusedId(),
                index     = index ? index : 200,
                groupName = _.escape(name);

            if (!id) {
                return;
            }

            var group = new RosterGroupModel({
                id: id,
                name: groupName,
                index: index
            });

            this.groups.add(group);

            return group;
        },

        updateGroup: function (group, name) {
            if (!group) {
                return;
            }

            group.set({
                name: name
            });

            return group;
        },

        deleteGroup: function (id) {
            var group = this.getGroup(id);

            if (group) {
                this.groups.remove(group);
            }

            return group;
        },

        setStatus: function (statusObj) {
            var id = statusObj.username,
                statuses = statusObj.status || {},
                buddyStatus = { status: 'offline' },
                buddyMessage = '',
                buddylist = this.groups.get(1),
                users = buddylist.users,
                sortOn = [ 'invisible', 'offline' ],
                existingUser = users.get(id),
                nickname = '',
                presenceBadge;

            if (!existingUser) {
                return;
            }

            var prevStatus = existingUser.get('status');
            if (prevStatus === statusObj.status) {
                return;
            }

            _.each(statuses, function (statusObj, resource) {
                if (statusObj.status === 'offline') {
                    buddyStatus = statusObj;
                    nickname = statusObj;
                }

                //if (resource.match(/\/oxmessenger$/)) {
                buddyStatus = statusObj;
                buddyMessage = statusObj.message;
                nickname = statusObj.nickname;
                //}

                if (!buddyStatus.capabilities) {
                    buddyStatus.capabilities.chat = true;
                    buddyStatus.capabilities.audio = false;
                    buddyStatus.capabilities.video = false;
                    buddyStatus.capabilities.mustFlash = false;
                    buddyStatus.capabilities.hasFlash = false;
                }
            });

            existingUser.set({
                canChat: buddyStatus.capabilities.chat,
                canAudio: buddyStatus.capabilities.audio,
                canVideo: buddyStatus.capabilities.video,
                mustFlash: buddyStatus.capabilities.mustFlash,
                hasFlash: buddyStatus.capabilities.hasFlash,
                priority: buddyStatus.priority,
                statusMessage: buddyStatus.message,
                nickname: nickname
            });

            existingUser.setStatus(buddyStatus.status, buddyStatus.message);

            // Only sort the list when a user comes or goes offline/invisible
            if (sortOn.indexOf(prevStatus) !== -1 || sortOn.indexOf(buddyStatus.status) !== -1) {
                this.groups.each(function (group) {
                    group.users.sort();
                });
            }

            // find drawn presence extension points and update the class
            if (buddyStatus.status === 'invisible') {
                UserPart.updatePresenceBadges(id, 'offline');
            } else {
                UserPart.updatePresenceBadges(id, buddyStatus.status);
            }
        },

        updatePresenceBadges: function (jid, status) {
            $('.messenger--presence[data-jid="' + jid + '"]')
                .removeClass('is-status--unknown ' +
                'is-status--offline ' +
                'is-status--online ' +
                'is-status--away ' +
                'is-status--dnd ' +
                'is-status--invisible ' +
                'is-status--xa')
                .addClass('is-status--' + status);
        },

        toggleHandler: function (event) {
            this.view.$el.css({
                    'top': $('#io-ox-banner:visible').outerHeight() + $('#io-ox-topbar:visible').outerHeight()
                })
                .toggleClass('messenger--is-hidden')
                .siblings('.messenger--roster-overlay').toggleClass('messenger--is-hidden');

            if (this.view.$el.is(':visible')) { // focus on search when roster is visible
                require('io.ox/core/notifications').hide();
                $('#roster-search').focus();
            }
        },

        offlineHandler: function (event) {
            // Force al roster users to offline
            var allUsers = this.getAllUsers();
            allUsers.each(function (user) {
                user.setStatus('offline');
            });
        },

        show: function () {
            this.view.$el.removeClass('messenger--is-hidden')
                .siblings('.messenger--roster-overlay').removeClass('messenger--is-hidden');
        },

        hide: function () {
            if (this.view) {
                this.view.$el.addClass('messenger--is-hidden')
                    .siblings('.messenger--roster-overlay').addClass('messenger--is-hidden');
            }
        }
    };

    return Roster;
});
